// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textautogeneratetextmcpprotocol_widgets_debug.h"


Q_LOGGING_CATEGORY(TEXTAUTOGENERATEMCPPROTOCOLWIDGETS_LOG, "org.kde.kf.TextAutoGenerateTextMcpProtocolWidgets", QtInfoMsg)

