/*
  SPDX-FileCopyrightText: 2025-2026 Laurent Montel <montel@kde.org>
  SPDX-FileCopyrightText: 2026 Florian Richer <florian.richer@protonmail.com>

  SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "plugincommonmodelinfosdelegateutils.h"

qreal PluginCommonModelInfosDelegateUtils::basicMargin()
{
    return 8;
}

int PluginCommonModelInfosDelegateUtils::textMargin()
{
    return 15;
}

qreal PluginCommonModelInfosDelegateUtils::rectRoundValue()
{
    return 5;
}

int PluginCommonModelInfosDelegateUtils::categoryOffset()
{
    return 10;
}

int PluginCommonModelInfosDelegateUtils::categoryMarginText()
{
    return 10;
}

int PluginCommonModelInfosDelegateUtils::spacingBetweenTextAndCategories()
{
    return 10;
}
