/*
  SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "configurepluginstreewidgetsearchlinewidget.h"
#include "configurepluginstreewidgetsearchline.h"

using namespace TextAddonsWidgets;

ConfigurePluginsTreeWidgetSearchLineWidget::ConfigurePluginsTreeWidgetSearchLineWidget(QWidget *parent, QTreeWidget *treeWidget)
    : KTreeWidgetSearchLineWidget(parent, treeWidget)
{
}

ConfigurePluginsTreeWidgetSearchLineWidget::~ConfigurePluginsTreeWidgetSearchLineWidget() = default;

KTreeWidgetSearchLine *ConfigurePluginsTreeWidgetSearchLineWidget::createSearchLine(QTreeWidget *treeWidget) const
{
    return new ConfigurePluginsTreeWidgetSearchLine(const_cast<ConfigurePluginsTreeWidgetSearchLineWidget *>(this), treeWidget);
}

#include "moc_configurepluginstreewidgetsearchlinewidget.cpp"
