/****************************************************************************
** Meta object code from reading C++ file 'plaintexteditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ktextaddons-2.1.2/textcustomeditor/plaintexteditor/plaintexteditor.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plaintexteditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16TextCustomEditor15PlainTextEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto TextCustomEditor::PlainTextEditor::qt_create_metaobjectdata<qt_meta_tag_ZN16TextCustomEditor15PlainTextEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextCustomEditor::PlainTextEditor",
        "findText",
        "",
        "replaceText",
        "spellCheckerAutoCorrect",
        "currentWord",
        "autoCorrectWord",
        "checkSpellingChanged",
        "languageChanged",
        "spellCheckStatus",
        "say",
        "text",
        "searchSupportChanged",
        "slotDisplayMessageIndicator",
        "message",
        "slotCheckSpelling",
        "slotSpeakText",
        "slotZoomReset",
        "searchSupport",
        "spellCheckingSupport",
        "textToSpeechSupport",
        "webShortcutSupport",
        "emojiSupport"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'findText'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replaceText'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'spellCheckerAutoCorrect'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'checkSpellingChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'languageChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'spellCheckStatus'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'say'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'searchSupportChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDisplayMessageIndicator'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'slotCheckSpelling'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSpeakText'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotZoomReset'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'searchSupport'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'spellCheckingSupport'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'textToSpeechSupport'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'webShortcutSupport'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'emojiSupport'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlainTextEditor, qt_meta_tag_ZN16TextCustomEditor15PlainTextEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextCustomEditor::PlainTextEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlainTextEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextCustomEditor15PlainTextEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextCustomEditor15PlainTextEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16TextCustomEditor15PlainTextEditorE_t>.metaTypes,
    nullptr
} };

void TextCustomEditor::PlainTextEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlainTextEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->findText(); break;
        case 1: _t->replaceText(); break;
        case 2: _t->spellCheckerAutoCorrect((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->checkSpellingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->languageChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->spellCheckStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->say((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->searchSupportChanged(); break;
        case 8: _t->slotDisplayMessageIndicator((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->slotCheckSpelling(); break;
        case 10: _t->slotSpeakText(); break;
        case 11: _t->slotZoomReset(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlainTextEditor::*)()>(_a, &PlainTextEditor::findText, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlainTextEditor::*)()>(_a, &PlainTextEditor::replaceText, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlainTextEditor::*)(const QString & , const QString & )>(_a, &PlainTextEditor::spellCheckerAutoCorrect, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlainTextEditor::*)(bool )>(_a, &PlainTextEditor::checkSpellingChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlainTextEditor::*)(const QString & )>(_a, &PlainTextEditor::languageChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlainTextEditor::*)(const QString & )>(_a, &PlainTextEditor::spellCheckStatus, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlainTextEditor::*)(const QString & )>(_a, &PlainTextEditor::say, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlainTextEditor::*)()>(_a, &PlainTextEditor::searchSupportChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->searchSupport(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->spellCheckingSupport(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->textToSpeechSupport(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->webShortcutSupport(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->emojiSupport(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSearchSupport(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setSpellCheckingSupport(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setTextToSpeechSupport(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setWebShortcutSupport(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setEmojiSupport(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *TextCustomEditor::PlainTextEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextCustomEditor::PlainTextEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextCustomEditor15PlainTextEditorE_t>.strings))
        return static_cast<void*>(this);
    return QPlainTextEdit::qt_metacast(_clname);
}

int TextCustomEditor::PlainTextEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlainTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void TextCustomEditor::PlainTextEditor::findText()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TextCustomEditor::PlainTextEditor::replaceText()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextCustomEditor::PlainTextEditor::spellCheckerAutoCorrect(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void TextCustomEditor::PlainTextEditor::checkSpellingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TextCustomEditor::PlainTextEditor::languageChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void TextCustomEditor::PlainTextEditor::spellCheckStatus(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TextCustomEditor::PlainTextEditor::say(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void TextCustomEditor::PlainTextEditor::searchSupportChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
