// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "autogeneratetext_mcpprotocolclientplugin_lib_debug.h"


Q_LOGGING_CATEGORY(AUTOGENERATETEXT_MCPPROTOCOLCLIENT_PLUGIN_LIB_LOG, "org.kde.kf.autogeneratetext.mcpprotocolclientplugin_lib", QtInfoMsg)

