/****************************************************************************
** Meta object code from reading C++ file 'textautogeneratelistviewdelegate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ktextaddons-2.1.2/textautogeneratetext/widgets/view/delegate/textautogeneratelistviewdelegate.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textautogeneratelistviewdelegate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20TextAutoGenerateText32TextAutoGenerateListViewDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto TextAutoGenerateText::TextAutoGenerateListViewDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN20TextAutoGenerateText32TextAutoGenerateListViewDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextAutoGenerateText::TextAutoGenerateListViewDelegate",
        "editMessage",
        "",
        "QModelIndex",
        "index",
        "copyMessage",
        "cancelRequested",
        "refreshRequested",
        "removeMessage",
        "textToSpeechRequested",
        "stopTextToSpeechRequested",
        "forkRequested",
        "updateView",
        "updateColors",
        "insertBlockCode",
        "str"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'editMessage'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'copyMessage'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'cancelRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'refreshRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'removeMessage'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'textToSpeechRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'stopTextToSpeechRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'forkRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'updateView'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateColors'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'insertBlockCode'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextAutoGenerateListViewDelegate, qt_meta_tag_ZN20TextAutoGenerateText32TextAutoGenerateListViewDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextAutoGenerateText::TextAutoGenerateListViewDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<TextAutoGenerateListViewMessageBaseDelegate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText32TextAutoGenerateListViewDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText32TextAutoGenerateListViewDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20TextAutoGenerateText32TextAutoGenerateListViewDelegateE_t>.metaTypes,
    nullptr
} };

void TextAutoGenerateText::TextAutoGenerateListViewDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextAutoGenerateListViewDelegate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->editMessage((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 1: _t->copyMessage((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 2: _t->cancelRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->refreshRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 4: _t->removeMessage((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 5: _t->textToSpeechRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 6: _t->stopTextToSpeechRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 7: _t->forkRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 8: _t->updateView(); break;
        case 9: _t->updateColors(); break;
        case 10: _t->insertBlockCode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)(const QModelIndex & )>(_a, &TextAutoGenerateListViewDelegate::editMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)(const QModelIndex & )>(_a, &TextAutoGenerateListViewDelegate::copyMessage, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)(const QModelIndex & )>(_a, &TextAutoGenerateListViewDelegate::cancelRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)(const QModelIndex & )>(_a, &TextAutoGenerateListViewDelegate::refreshRequested, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)(const QModelIndex & )>(_a, &TextAutoGenerateListViewDelegate::removeMessage, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)(const QModelIndex & )>(_a, &TextAutoGenerateListViewDelegate::textToSpeechRequested, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)(const QModelIndex & )>(_a, &TextAutoGenerateListViewDelegate::stopTextToSpeechRequested, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)(const QModelIndex & )>(_a, &TextAutoGenerateListViewDelegate::forkRequested, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)()>(_a, &TextAutoGenerateListViewDelegate::updateView, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)()>(_a, &TextAutoGenerateListViewDelegate::updateColors, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateListViewDelegate::*)(const QString & )>(_a, &TextAutoGenerateListViewDelegate::insertBlockCode, 10))
            return;
    }
}

const QMetaObject *TextAutoGenerateText::TextAutoGenerateListViewDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextAutoGenerateText::TextAutoGenerateListViewDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText32TextAutoGenerateListViewDelegateE_t>.strings))
        return static_cast<void*>(this);
    return TextAutoGenerateListViewMessageBaseDelegate::qt_metacast(_clname);
}

int TextAutoGenerateText::TextAutoGenerateListViewDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TextAutoGenerateListViewMessageBaseDelegate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::editMessage(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::copyMessage(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::cancelRequested(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::refreshRequested(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::removeMessage(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::textToSpeechRequested(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::stopTextToSpeechRequested(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::forkRequested(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::updateView()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::updateColors()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void TextAutoGenerateText::TextAutoGenerateListViewDelegate::insertBlockCode(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
