/*
  SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/
#pragma once
#include "textautogeneratelmstudio_export.h"
#include <QString>

namespace LMStudioCommonUtils
{
[[nodiscard]] TEXTAUTOGENERATELMSTUDIO_EXPORT QString responsesPath();
[[nodiscard]] TEXTAUTOGENERATELMSTUDIO_EXPORT QString modelsPath();
[[nodiscard]] TEXTAUTOGENERATELMSTUDIO_EXPORT QString completionPath();
};
