// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "autogeneratetext_llamacpp_generate_json_debug.h"


Q_LOGGING_CATEGORY(AUTOGENERATETEXT_LMSTUDIO_GENERATE_JSON_LOG, "org.kde.kf.autogeneratetext.llamacpp_json", QtInfoMsg)

