/****************************************************************************
** Meta object code from reading C++ file 'textfindreplacewidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ktextaddons-2.1.2/textcustomeditor/widgets/textfindreplacewidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textfindreplacewidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16TextCustomEditor14TextFindWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto TextCustomEditor::TextFindWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16TextCustomEditor14TextFindWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextCustomEditor::TextFindWidget",
        "findNext",
        "",
        "findPrev",
        "clearSearch",
        "autoSearch",
        "updateSearchOptions",
        "searchStringEmpty"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'findNext'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'findPrev'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clearSearch'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoSearch'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'updateSearchOptions'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'searchStringEmpty'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextFindWidget, qt_meta_tag_ZN16TextCustomEditor14TextFindWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextCustomEditor::TextFindWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextCustomEditor14TextFindWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextCustomEditor14TextFindWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16TextCustomEditor14TextFindWidgetE_t>.metaTypes,
    nullptr
} };

void TextCustomEditor::TextFindWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextFindWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->findNext(); break;
        case 1: _t->findPrev(); break;
        case 2: _t->clearSearch(); break;
        case 3: _t->autoSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->updateSearchOptions(); break;
        case 5: _t->searchStringEmpty((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextFindWidget::*)()>(_a, &TextFindWidget::findNext, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextFindWidget::*)()>(_a, &TextFindWidget::findPrev, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextFindWidget::*)()>(_a, &TextFindWidget::clearSearch, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextFindWidget::*)(const QString & )>(_a, &TextFindWidget::autoSearch, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextFindWidget::*)()>(_a, &TextFindWidget::updateSearchOptions, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextFindWidget::*)(bool )>(_a, &TextFindWidget::searchStringEmpty, 5))
            return;
    }
}

const QMetaObject *TextCustomEditor::TextFindWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextCustomEditor::TextFindWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextCustomEditor14TextFindWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TextCustomEditor::TextFindWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void TextCustomEditor::TextFindWidget::findNext()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TextCustomEditor::TextFindWidget::findPrev()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextCustomEditor::TextFindWidget::clearSearch()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TextCustomEditor::TextFindWidget::autoSearch(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TextCustomEditor::TextFindWidget::updateSearchOptions()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TextCustomEditor::TextFindWidget::searchStringEmpty(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN16TextCustomEditor17TextReplaceWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto TextCustomEditor::TextReplaceWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16TextCustomEditor17TextReplaceWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextCustomEditor::TextReplaceWidget",
        "replaceText",
        "",
        "replaceAllText",
        "slotSearchStringEmpty"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'replaceText'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replaceAllText'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSearchStringEmpty'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextReplaceWidget, qt_meta_tag_ZN16TextCustomEditor17TextReplaceWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextCustomEditor::TextReplaceWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextCustomEditor17TextReplaceWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextCustomEditor17TextReplaceWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16TextCustomEditor17TextReplaceWidgetE_t>.metaTypes,
    nullptr
} };

void TextCustomEditor::TextReplaceWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextReplaceWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->replaceText(); break;
        case 1: _t->replaceAllText(); break;
        case 2: _t->slotSearchStringEmpty((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextReplaceWidget::*)()>(_a, &TextReplaceWidget::replaceText, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextReplaceWidget::*)()>(_a, &TextReplaceWidget::replaceAllText, 1))
            return;
    }
}

const QMetaObject *TextCustomEditor::TextReplaceWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextCustomEditor::TextReplaceWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TextCustomEditor17TextReplaceWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TextCustomEditor::TextReplaceWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void TextCustomEditor::TextReplaceWidget::replaceText()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TextCustomEditor::TextReplaceWidget::replaceAllText()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
