/****************************************************************************
** Meta object code from reading C++ file 'textautogeneratetextplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ktextaddons-2.1.2/textautogeneratetext/core/textautogeneratetextplugin.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textautogeneratetextplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20TextAutoGenerateText26TextAutoGenerateTextPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto TextAutoGenerateText::TextAutoGenerateTextPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN20TextAutoGenerateText26TextAutoGenerateTextPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextAutoGenerateText::TextAutoGenerateTextPlugin",
        "downloadErrorOccurred",
        "",
        "message",
        "initializedDone",
        "askToAssistantAnswer",
        "TextAutoGenerateText::TextAutoGenerateReply::Response",
        "askToAssistantDone",
        "configChanged",
        "downloadModelFinished",
        "modelName",
        "callTools",
        "QList<TextAutoGenerateText::TextAutoGenerateReply::ToolCallArgumen"
        "tInfo>",
        "info",
        "currentModelChanged",
        "errorOccurred",
        "QNetworkReply::NetworkError",
        "e",
        "errorString",
        "processStartSuccessed",
        "processStartFailed",
        "errorStr"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'downloadErrorOccurred'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'initializedDone'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'askToAssistantAnswer'
        QtMocHelpers::SignalData<void(const TextAutoGenerateText::TextAutoGenerateReply::Response &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 3 },
        }}),
        // Signal 'askToAssistantDone'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'downloadModelFinished'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'callTools'
        QtMocHelpers::SignalData<void(const QList<TextAutoGenerateText::TextAutoGenerateReply::ToolCallArgumentInfo> &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'currentModelChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QNetworkReply::NetworkError, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'processStartSuccessed'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'processStartFailed'
        QtMocHelpers::SignalData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextAutoGenerateTextPlugin, qt_meta_tag_ZN20TextAutoGenerateText26TextAutoGenerateTextPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextAutoGenerateText::TextAutoGenerateTextPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText26TextAutoGenerateTextPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText26TextAutoGenerateTextPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20TextAutoGenerateText26TextAutoGenerateTextPluginE_t>.metaTypes,
    nullptr
} };

void TextAutoGenerateText::TextAutoGenerateTextPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextAutoGenerateTextPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->downloadErrorOccurred((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->initializedDone(); break;
        case 2: _t->askToAssistantAnswer((*reinterpret_cast<std::add_pointer_t<TextAutoGenerateText::TextAutoGenerateReply::Response>>(_a[1]))); break;
        case 3: _t->askToAssistantDone(); break;
        case 4: _t->configChanged(); break;
        case 5: _t->downloadModelFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->callTools((*reinterpret_cast<std::add_pointer_t<QList<TextAutoGenerateText::TextAutoGenerateReply::ToolCallArgumentInfo>>>(_a[1]))); break;
        case 7: _t->currentModelChanged(); break;
        case 8: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->processStartSuccessed(); break;
        case 10: _t->processStartFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)(const QString & )>(_a, &TextAutoGenerateTextPlugin::downloadErrorOccurred, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)()>(_a, &TextAutoGenerateTextPlugin::initializedDone, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)(const TextAutoGenerateText::TextAutoGenerateReply::Response & )>(_a, &TextAutoGenerateTextPlugin::askToAssistantAnswer, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)()>(_a, &TextAutoGenerateTextPlugin::askToAssistantDone, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)()>(_a, &TextAutoGenerateTextPlugin::configChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)(const QString & )>(_a, &TextAutoGenerateTextPlugin::downloadModelFinished, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)(const QList<TextAutoGenerateText::TextAutoGenerateReply::ToolCallArgumentInfo> & )>(_a, &TextAutoGenerateTextPlugin::callTools, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)()>(_a, &TextAutoGenerateTextPlugin::currentModelChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)(QNetworkReply::NetworkError , const QString & )>(_a, &TextAutoGenerateTextPlugin::errorOccurred, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)()>(_a, &TextAutoGenerateTextPlugin::processStartSuccessed, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateTextPlugin::*)(const QString & )>(_a, &TextAutoGenerateTextPlugin::processStartFailed, 10))
            return;
    }
}

const QMetaObject *TextAutoGenerateText::TextAutoGenerateTextPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextAutoGenerateText::TextAutoGenerateTextPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText26TextAutoGenerateTextPluginE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TextAutoGenerateText::TextAutoGenerateTextPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void TextAutoGenerateText::TextAutoGenerateTextPlugin::downloadErrorOccurred(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TextAutoGenerateText::TextAutoGenerateTextPlugin::initializedDone()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextAutoGenerateText::TextAutoGenerateTextPlugin::askToAssistantAnswer(const TextAutoGenerateText::TextAutoGenerateReply::Response & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void TextAutoGenerateText::TextAutoGenerateTextPlugin::askToAssistantDone()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TextAutoGenerateText::TextAutoGenerateTextPlugin::configChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TextAutoGenerateText::TextAutoGenerateTextPlugin::downloadModelFinished(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TextAutoGenerateText::TextAutoGenerateTextPlugin::callTools(const QList<TextAutoGenerateText::TextAutoGenerateReply::ToolCallArgumentInfo> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void TextAutoGenerateText::TextAutoGenerateTextPlugin::currentModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void TextAutoGenerateText::TextAutoGenerateTextPlugin::errorOccurred(QNetworkReply::NetworkError _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void TextAutoGenerateText::TextAutoGenerateTextPlugin::processStartSuccessed()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void TextAutoGenerateText::TextAutoGenerateTextPlugin::processStartFailed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
