/*
  SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "textautogeneratetexttoolinternaljob.h"

using namespace TextAutoGenerateText;
TextAutoGenerateTextToolInternalJob::TextAutoGenerateTextToolInternalJob(QObject *parent)
    : TextAutoGenerateTextToolBaseJob(parent)
{
}

TextAutoGenerateTextToolInternalJob::~TextAutoGenerateTextToolInternalJob() = default;

void TextAutoGenerateTextToolInternalJob::start()
{
    Q_ASSERT(false);
}

QList<TextAutoGenerateTextToolInternalProperty> TextAutoGenerateTextToolInternalJob::properties() const
{
    return mProperties;
}

void TextAutoGenerateTextToolInternalJob::setProperties(const QList<TextAutoGenerateText::TextAutoGenerateTextToolInternalProperty> &newProperties)
{
    mProperties = newProperties;
}

QDebug operator<<(QDebug d, const TextAutoGenerateText::TextAutoGenerateTextToolInternalJob::TextToolPluginInfo &t)
{
    d.space() << "content" << t.content;
    d.space() << "messageUuid" << t.messageUuid;
    d.space() << "chatId" << t.chatId;
    d.space() << "toolIdentifier" << t.toolIdentifier;
    d.space() << "attachementInfoList" << t.attachementInfoList;
    return d;
}

#include "moc_textautogeneratetexttoolinternaljob.cpp"
