/*
   SPDX-FileCopyrightText: 2025-2026 Laurent Montel <montel@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once

#include <QItemDelegate>
namespace TextAutoGenerateText
{
class TextAutoGenerateMenuListViewDelegate : public QItemDelegate
{
    Q_OBJECT
public:
    explicit TextAutoGenerateMenuListViewDelegate(QObject *parent = nullptr);
    ~TextAutoGenerateMenuListViewDelegate() override;

    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const override;

    void setEditorData(QWidget *editor, const QModelIndex &index) const override;

    void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const override;

    void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
};
}
