/*
  SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/
#include "addmcpprotocolclientwidget.h"
#include <QVBoxLayout>
using namespace TextAutoGenerateTextMcpProtocolWidgets;
AddMcpProtocolClientWidget::AddMcpProtocolClientWidget(QWidget *parent)
    : QWidget{parent}
{
}

AddMcpProtocolClientWidget::~AddMcpProtocolClientWidget() = default;

#include "moc_addmcpprotocolclientwidget.cpp"
