/****************************************************************************
** Meta object code from reading C++ file 'textautogenerateresultwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ktextaddons-2.1.2/textautogeneratetext/widgets/common/textautogenerateresultwidget.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textautogenerateresultwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20TextAutoGenerateText28TextAutoGenerateResultWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto TextAutoGenerateText::TextAutoGenerateResultWidget::qt_create_metaobjectdata<qt_meta_tag_ZN20TextAutoGenerateText28TextAutoGenerateResultWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextAutoGenerateText::TextAutoGenerateResultWidget",
        "editMessageRequested",
        "",
        "QModelIndex",
        "index",
        "QList<QByteArray>",
        "tools",
        "cancelRequested",
        "uuid",
        "refreshAnswerRequested",
        "charId",
        "indexAnswer",
        "QList<TextAutoGenerateText::TextAutoGenerateAttachmentUtils::Attac"
        "hmentElementInfo>",
        "attachmentInfoList",
        "switchToChat",
        "closeSearchBarRequested"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'editMessageRequested'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QList<QByteArray> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'cancelRequested'
        QtMocHelpers::SignalData<void(const QByteArray &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 8 },
        }}),
        // Signal 'refreshAnswerRequested'
        QtMocHelpers::SignalData<void(const QByteArray &, const QModelIndex &, const QList<QByteArray> &, const QList<TextAutoGenerateText::TextAutoGenerateAttachmentUtils::AttachmentElementInfo> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 10 }, { 0x80000000 | 3, 11 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 12, 13 },
        }}),
        // Signal 'switchToChat'
        QtMocHelpers::SignalData<void(const QByteArray &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 8 },
        }}),
        // Signal 'closeSearchBarRequested'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextAutoGenerateResultWidget, qt_meta_tag_ZN20TextAutoGenerateText28TextAutoGenerateResultWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextAutoGenerateText::TextAutoGenerateResultWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText28TextAutoGenerateResultWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText28TextAutoGenerateResultWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20TextAutoGenerateText28TextAutoGenerateResultWidgetE_t>.metaTypes,
    nullptr
} };

void TextAutoGenerateText::TextAutoGenerateResultWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextAutoGenerateResultWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->editMessageRequested((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QByteArray>>>(_a[2]))); break;
        case 1: _t->cancelRequested((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 2: _t->refreshAnswerRequested((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<QByteArray>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QList<TextAutoGenerateText::TextAutoGenerateAttachmentUtils::AttachmentElementInfo>>>(_a[4]))); break;
        case 3: _t->switchToChat((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 4: _t->closeSearchBarRequested(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QByteArray> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QByteArray> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateResultWidget::*)(const QModelIndex & , const QList<QByteArray> & )>(_a, &TextAutoGenerateResultWidget::editMessageRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateResultWidget::*)(const QByteArray & )>(_a, &TextAutoGenerateResultWidget::cancelRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateResultWidget::*)(const QByteArray & , const QModelIndex & , const QList<QByteArray> & , const QList<TextAutoGenerateText::TextAutoGenerateAttachmentUtils::AttachmentElementInfo> & )>(_a, &TextAutoGenerateResultWidget::refreshAnswerRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateResultWidget::*)(const QByteArray & )>(_a, &TextAutoGenerateResultWidget::switchToChat, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateResultWidget::*)()>(_a, &TextAutoGenerateResultWidget::closeSearchBarRequested, 4))
            return;
    }
}

const QMetaObject *TextAutoGenerateText::TextAutoGenerateResultWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextAutoGenerateText::TextAutoGenerateResultWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText28TextAutoGenerateResultWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TextAutoGenerateText::TextAutoGenerateResultWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void TextAutoGenerateText::TextAutoGenerateResultWidget::editMessageRequested(const QModelIndex & _t1, const QList<QByteArray> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void TextAutoGenerateText::TextAutoGenerateResultWidget::cancelRequested(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TextAutoGenerateText::TextAutoGenerateResultWidget::refreshAnswerRequested(const QByteArray & _t1, const QModelIndex & _t2, const QList<QByteArray> & _t3, const QList<TextAutoGenerateText::TextAutoGenerateAttachmentUtils::AttachmentElementInfo> & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 3
void TextAutoGenerateText::TextAutoGenerateResultWidget::switchToChat(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TextAutoGenerateText::TextAutoGenerateResultWidget::closeSearchBarRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
