/*
  SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/
#pragma once

#include "common/mcpbase.h"
#include <QObject>
class QProcess;
class McpClientStdioPluginInterface;
class McpClientStdio : public TextAutoGenerateTextMcpProtocolCore::McpBase
{
    Q_OBJECT
public:
    explicit McpClientStdio(McpClientStdioPluginInterface *interface, QObject *parent = nullptr);
    ~McpClientStdio() override;

    void connection() override;
    void send(const QJsonObject &obj) override;

private:
    QProcess *const mProcess;
    McpClientStdioPluginInterface *const mInterface;
};
