/*
  SPDX-FileCopyrightText: 2025-2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/
#pragma once
#include "textautogeneratetext_private_export.h"
#include <KSyntaxHighlighting/Repository>
#include <QWidget>
class QTextEdit;
class QJsonObject;
class QJsonArray;
namespace TextAutoGenerateText
{
class TEXTAUTOGENERATETEXT_TESTS_EXPORT TextAutoGenerateToolShowMetaDataWidget : public QWidget
{
    Q_OBJECT
public:
    explicit TextAutoGenerateToolShowMetaDataWidget(QWidget *parent = nullptr);
    ~TextAutoGenerateToolShowMetaDataWidget() override;

    void setMetaData(const QJsonObject &obj);
    void setMetaData(const QJsonArray &array);

private:
    KSyntaxHighlighting::Repository mRepo;
    QTextEdit *const mTextEdit;
};
}
