/****************************************************************************
** Meta object code from reading C++ file 'textautogeneratemanagerbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ktextaddons-2.1.2/textautogeneratetext/core/textautogeneratemanagerbase.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textautogeneratemanagerbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20TextAutoGenerateText27TextAutoGenerateManagerBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto TextAutoGenerateText::TextAutoGenerateManagerBase::qt_create_metaobjectdata<qt_meta_tag_ZN20TextAutoGenerateText27TextAutoGenerateManagerBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextAutoGenerateText::TextAutoGenerateManagerBase",
        "modelsLoadDone",
        "",
        "TextAutoGenerateText::TextAutoGenerateManagerBase::ModelsInfo",
        "models",
        "refreshInstalledModels",
        "finished",
        "TextAutoGenerateText::TextAutoGenerateReply::Response",
        "replyText",
        "downloadDone",
        "modelName",
        "canceled",
        "showModelInfoDone",
        "errorOccurred",
        "QNetworkReply::NetworkError",
        "e",
        "errorString"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modelsLoadDone'
        QtMocHelpers::SignalData<void(const TextAutoGenerateText::TextAutoGenerateManagerBase::ModelsInfo &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'refreshInstalledModels'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(const TextAutoGenerateText::TextAutoGenerateReply::Response &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'downloadDone'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'canceled'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'showModelInfoDone'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QNetworkReply::NetworkError, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::QString, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextAutoGenerateManagerBase, qt_meta_tag_ZN20TextAutoGenerateText27TextAutoGenerateManagerBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextAutoGenerateText::TextAutoGenerateManagerBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText27TextAutoGenerateManagerBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText27TextAutoGenerateManagerBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20TextAutoGenerateText27TextAutoGenerateManagerBaseE_t>.metaTypes,
    nullptr
} };

void TextAutoGenerateText::TextAutoGenerateManagerBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextAutoGenerateManagerBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modelsLoadDone((*reinterpret_cast<std::add_pointer_t<TextAutoGenerateText::TextAutoGenerateManagerBase::ModelsInfo>>(_a[1]))); break;
        case 1: _t->refreshInstalledModels(); break;
        case 2: _t->finished((*reinterpret_cast<std::add_pointer_t<TextAutoGenerateText::TextAutoGenerateReply::Response>>(_a[1]))); break;
        case 3: _t->downloadDone((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->canceled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->showModelInfoDone((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManagerBase::*)(const TextAutoGenerateText::TextAutoGenerateManagerBase::ModelsInfo & )>(_a, &TextAutoGenerateManagerBase::modelsLoadDone, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManagerBase::*)()>(_a, &TextAutoGenerateManagerBase::refreshInstalledModels, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManagerBase::*)(const TextAutoGenerateText::TextAutoGenerateReply::Response & )>(_a, &TextAutoGenerateManagerBase::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManagerBase::*)(const QString & )>(_a, &TextAutoGenerateManagerBase::downloadDone, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManagerBase::*)(const QString & )>(_a, &TextAutoGenerateManagerBase::canceled, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManagerBase::*)(const QString & )>(_a, &TextAutoGenerateManagerBase::showModelInfoDone, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManagerBase::*)(QNetworkReply::NetworkError , const QString & )>(_a, &TextAutoGenerateManagerBase::errorOccurred, 6))
            return;
    }
}

const QMetaObject *TextAutoGenerateText::TextAutoGenerateManagerBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextAutoGenerateText::TextAutoGenerateManagerBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText27TextAutoGenerateManagerBaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TextAutoGenerateText::TextAutoGenerateManagerBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void TextAutoGenerateText::TextAutoGenerateManagerBase::modelsLoadDone(const TextAutoGenerateText::TextAutoGenerateManagerBase::ModelsInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TextAutoGenerateText::TextAutoGenerateManagerBase::refreshInstalledModels()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextAutoGenerateText::TextAutoGenerateManagerBase::finished(const TextAutoGenerateText::TextAutoGenerateReply::Response & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void TextAutoGenerateText::TextAutoGenerateManagerBase::downloadDone(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TextAutoGenerateText::TextAutoGenerateManagerBase::canceled(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void TextAutoGenerateText::TextAutoGenerateManagerBase::showModelInfoDone(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TextAutoGenerateText::TextAutoGenerateManagerBase::errorOccurred(QNetworkReply::NetworkError _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}
QT_WARNING_POP
