/*
  SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "ollamacloudsettings.h"

using namespace Qt::Literals::StringLiterals;
OllamaCloudSettings::OllamaCloudSettings() = default;

OllamaCloudSettings::~OllamaCloudSettings() = default;

void OllamaCloudSettings::load(const KConfigGroup &config)
{
    OllamaCommonSettings::load(config);
}

void OllamaCloudSettings::save(KConfigGroup &config)
{
    OllamaCommonSettings::save(config);
}
