// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textautogeneratetextwidget_menu_debug.h"


Q_LOGGING_CATEGORY(TEXTAUTOGENERATETEXT_WIDGET_MENU_LOG, "org.kde.kf.textautogeneratetext.widget_menu", QtInfoMsg)

