/****************************************************************************
** Meta object code from reading C++ file 'richtextquicktextformat.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ktextaddons-2.1.2/textaddonswidgets/richtextquicktextformat.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'richtextquicktextformat.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17TextAddonsWidgets23RichTextQuickTextFormatE_t {};
} // unnamed namespace

template <> constexpr inline auto TextAddonsWidgets::RichTextQuickTextFormat::qt_create_metaobjectdata<qt_meta_tag_ZN17TextAddonsWidgets23RichTextQuickTextFormatE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextAddonsWidgets::RichTextQuickTextFormat",
        "quickTextFormatRequested",
        "",
        "TextAddonsWidgets::RichTextQuickTextFormat::QuickTextFormatType",
        "type",
        "QuickTextFormatType",
        "Unknown",
        "Bold",
        "Italic",
        "StrikeThrough",
        "CodeBlock",
        "BlockQuote",
        "InsertLink",
        "UnderLine",
        "QuickTextFormatTypes"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'quickTextFormatRequested'
        QtMocHelpers::SignalData<void(TextAddonsWidgets::RichTextQuickTextFormat::QuickTextFormatType)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'QuickTextFormatType'
        QtMocHelpers::EnumData<enum QuickTextFormatType>(5, 5, QMC::EnumIsScoped).add({
            {    6, QuickTextFormatType::Unknown },
            {    7, QuickTextFormatType::Bold },
            {    8, QuickTextFormatType::Italic },
            {    9, QuickTextFormatType::StrikeThrough },
            {   10, QuickTextFormatType::CodeBlock },
            {   11, QuickTextFormatType::BlockQuote },
            {   12, QuickTextFormatType::InsertLink },
            {   13, QuickTextFormatType::UnderLine },
        }),
        // flag 'QuickTextFormatTypes'
        QtMocHelpers::EnumData<QuickTextFormatTypes>(14, 5, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {    6, QuickTextFormatType::Unknown },
            {    7, QuickTextFormatType::Bold },
            {    8, QuickTextFormatType::Italic },
            {    9, QuickTextFormatType::StrikeThrough },
            {   10, QuickTextFormatType::CodeBlock },
            {   11, QuickTextFormatType::BlockQuote },
            {   12, QuickTextFormatType::InsertLink },
            {   13, QuickTextFormatType::UnderLine },
        }),
    };
    return QtMocHelpers::metaObjectData<RichTextQuickTextFormat, qt_meta_tag_ZN17TextAddonsWidgets23RichTextQuickTextFormatE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextAddonsWidgets::RichTextQuickTextFormat::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TextAddonsWidgets23RichTextQuickTextFormatE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TextAddonsWidgets23RichTextQuickTextFormatE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17TextAddonsWidgets23RichTextQuickTextFormatE_t>.metaTypes,
    nullptr
} };

void TextAddonsWidgets::RichTextQuickTextFormat::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RichTextQuickTextFormat *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->quickTextFormatRequested((*reinterpret_cast<std::add_pointer_t<TextAddonsWidgets::RichTextQuickTextFormat::QuickTextFormatType>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RichTextQuickTextFormat::*)(TextAddonsWidgets::RichTextQuickTextFormat::QuickTextFormatType )>(_a, &RichTextQuickTextFormat::quickTextFormatRequested, 0))
            return;
    }
}

const QMetaObject *TextAddonsWidgets::RichTextQuickTextFormat::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextAddonsWidgets::RichTextQuickTextFormat::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TextAddonsWidgets23RichTextQuickTextFormatE_t>.strings))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int TextAddonsWidgets::RichTextQuickTextFormat::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void TextAddonsWidgets::RichTextQuickTextFormat::quickTextFormatRequested(TextAddonsWidgets::RichTextQuickTextFormat::QuickTextFormatType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
