// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "autogeneratetext_plugincommon_debug.h"


Q_LOGGING_CATEGORY(AUTOGENERATETEXT_PLUGINCOMMON_LOG, "org.kde.kf.autogeneratetext.plugincommon", QtInfoMsg)

