// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textemoticonswidgets_debug.h"


Q_LOGGING_CATEGORY(TEXTEMOTICONSWIDGETS_LOG, "org.kde.kf.textemoticonswidgets", QtInfoMsg)

