// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textautogeneratetextcore_toolplugin_debug.h"


Q_LOGGING_CATEGORY(TEXTAUTOGENERATETEXT_CORE_TOOL_PLUGIN_LOG, "org.kde.kf.textautogeneratetext.core.tool.plugin", QtInfoMsg)

