// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "bingtranslator_debug.h"


Q_LOGGING_CATEGORY(TRANSLATOR_BING_LOG, "org.kde.kf.texttranslator.bing", QtInfoMsg)

