/********************************************************************************
** Form generated from reading UI file 'autocorrectionwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_AUTOCORRECTIONWIDGET_H
#define UI_AUTOCORRECTIONWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "autocorrectionlanguage.h"
#include "autocorrectionlistwidget.h"
#include "autocorrectiontreewidget.h"

QT_BEGIN_NAMESPACE

class Ui_AutoCorrectionWidget
{
public:
    QGridLayout *gridLayout;
    QCheckBox *enabledAutocorrection;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    TextAutoCorrectionWidgets::AutoCorrectionLanguage *autocorrectionLanguage;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *exportAutoCorrection;
    QPushButton *importAutoCorrection;
    QSpacerItem *horizontalSpacer;
    QTabWidget *tabWidget;
    QWidget *tab_3;
    QGridLayout *_8;
    QVBoxLayout *_11;
    QPushButton *addButton;
    QPushButton *removeButton;
    QSpacerItem *spacerItem;
    QCheckBox *advancedAutocorrection;
    QHBoxLayout *_10;
    QLabel *findLabel;
    QLineEdit *find;
    QLabel *replaceLabel;
    QLineEdit *replace;
    TextAutoCorrectionWidgets::AutoCorrectionTreeWidget *treeWidget;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_3;
    QLabel *label_4;
    QLineEdit *abbreviation;
    QVBoxLayout *_14;
    QPushButton *add1;
    QPushButton *remove1;
    QSpacerItem *spacerItem1;
    TextAutoCorrectionWidgets::AutoCorrectionListWidget *abbreviationList;
    QGridLayout *gridLayout_2;
    QLabel *label_5;
    QLineEdit *twoUpperLetter;
    QVBoxLayout *_16;
    QPushButton *add2;
    QPushButton *remove2;
    QSpacerItem *spacerItem2;
    TextAutoCorrectionWidgets::AutoCorrectionListWidget *twoUpperLetterList;
    QWidget *tab_1;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *upperCase;
    QCheckBox *upperUpper;
    QCheckBox *ignoreDoubleSpace;
    QCheckBox *autoReplaceNumber;
    QCheckBox *capitalizeDaysName;
    QCheckBox *autoFormatUrl;
    QCheckBox *autoChangeFormat;
    QCheckBox *autoSuperScript;
    QCheckBox *addNonBreakingSpaceInFrench;
    QCheckBox *replaceDoubleQuotesByFrenchQuotes;
    QSpacerItem *spacerItem3;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_5;
    QCheckBox *typographicDoubleQuotes;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout_4;
    QLabel *doubleQuoteBeginReplace;
    QPushButton *doubleQuote1;
    QPushButton *doubleQuote2;
    QLabel *label_3;
    QLabel *doubleQuoteEndReplace;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *doubleDefault;
    QSpacerItem *horizontalSpacer_3;
    QCheckBox *typographicSingleQuotes;
    QVBoxLayout *verticalLayout_4;
    QGridLayout *gridLayout_5;
    QSpacerItem *horizontalSpacer_5;
    QPushButton *singleQuote1;
    QPushButton *singleQuote2;
    QLabel *label_8;
    QLabel *simpleQuoteBeginReplace;
    QLabel *label_9;
    QLabel *simpleQuoteEndReplace;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *singleDefault;
    QSpacerItem *horizontalSpacer_4;
    QSpacerItem *spacerItem4;
    QWidget *tab;
    QGridLayout *gridLayout_6;
    QLabel *label_6;
    QLabel *systemPath;
    QSpacerItem *horizontalSpacer_6;
    QLabel *label_7;
    QLabel *writablePath;
    QSpacerItem *horizontalSpacer_7;
    QLabel *label_10;
    QHBoxLayout *horizontalLayout_6;
    QLineEdit *customSystemPath;
    QToolButton *customSystemPathToolButton;
    QLabel *label_11;
    QHBoxLayout *horizontalLayout_5;
    QLineEdit *customWritablePath;
    QToolButton *customWritablePathToolButton;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *AutoCorrectionWidget)
    {
        if (AutoCorrectionWidget->objectName().isEmpty())
            AutoCorrectionWidget->setObjectName("AutoCorrectionWidget");
        AutoCorrectionWidget->resize(741, 572);
        gridLayout = new QGridLayout(AutoCorrectionWidget);
        gridLayout->setObjectName("gridLayout");
        enabledAutocorrection = new QCheckBox(AutoCorrectionWidget);
        enabledAutocorrection->setObjectName("enabledAutocorrection");

        gridLayout->addWidget(enabledAutocorrection, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(AutoCorrectionWidget);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        autocorrectionLanguage = new TextAutoCorrectionWidgets::AutoCorrectionLanguage(AutoCorrectionWidget);
        autocorrectionLanguage->setObjectName("autocorrectionLanguage");

        horizontalLayout->addWidget(autocorrectionLanguage);


        gridLayout->addLayout(horizontalLayout, 1, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        exportAutoCorrection = new QPushButton(AutoCorrectionWidget);
        exportAutoCorrection->setObjectName("exportAutoCorrection");

        horizontalLayout_2->addWidget(exportAutoCorrection);

        importAutoCorrection = new QPushButton(AutoCorrectionWidget);
        importAutoCorrection->setObjectName("importAutoCorrection");

        horizontalLayout_2->addWidget(importAutoCorrection);

        horizontalSpacer = new QSpacerItem(469, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        gridLayout->addLayout(horizontalLayout_2, 3, 0, 1, 1);

        tabWidget = new QTabWidget(AutoCorrectionWidget);
        tabWidget->setObjectName("tabWidget");
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        _8 = new QGridLayout(tab_3);
        _8->setObjectName("_8");
        _11 = new QVBoxLayout();
        _11->setObjectName("_11");
        addButton = new QPushButton(tab_3);
        addButton->setObjectName("addButton");
        addButton->setEnabled(false);

        _11->addWidget(addButton);

        removeButton = new QPushButton(tab_3);
        removeButton->setObjectName("removeButton");
        removeButton->setEnabled(false);

        _11->addWidget(removeButton);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _11->addItem(spacerItem);


        _8->addLayout(_11, 1, 1, 2, 1);

        advancedAutocorrection = new QCheckBox(tab_3);
        advancedAutocorrection->setObjectName("advancedAutocorrection");

        _8->addWidget(advancedAutocorrection, 0, 0, 1, 1);

        _10 = new QHBoxLayout();
        _10->setObjectName("_10");
        findLabel = new QLabel(tab_3);
        findLabel->setObjectName("findLabel");

        _10->addWidget(findLabel);

        find = new QLineEdit(tab_3);
        find->setObjectName("find");
        find->setClearButtonEnabled(true);
        find->setProperty("showClearButton", QVariant(false));

        _10->addWidget(find);

        replaceLabel = new QLabel(tab_3);
        replaceLabel->setObjectName("replaceLabel");

        _10->addWidget(replaceLabel);

        replace = new QLineEdit(tab_3);
        replace->setObjectName("replace");
        replace->setClearButtonEnabled(true);
        replace->setProperty("showClearButton", QVariant(false));

        _10->addWidget(replace);


        _8->addLayout(_10, 1, 0, 1, 1);

        treeWidget = new TextAutoCorrectionWidgets::AutoCorrectionTreeWidget(tab_3);
        treeWidget->setObjectName("treeWidget");
        treeWidget->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        treeWidget->setAlternatingRowColors(true);
        treeWidget->setSelectionMode(QAbstractItemView::SelectionMode::ExtendedSelection);
        treeWidget->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        treeWidget->setRootIsDecorated(false);

        _8->addWidget(treeWidget, 2, 0, 1, 1);

        tabWidget->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        verticalLayout = new QVBoxLayout(tab_4);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        label_4 = new QLabel(tab_4);
        label_4->setObjectName("label_4");

        gridLayout_3->addWidget(label_4, 0, 0, 1, 2);

        abbreviation = new QLineEdit(tab_4);
        abbreviation->setObjectName("abbreviation");
        abbreviation->setClearButtonEnabled(true);
        abbreviation->setProperty("showClearButton", QVariant(false));

        gridLayout_3->addWidget(abbreviation, 1, 0, 1, 1);

        _14 = new QVBoxLayout();
        _14->setObjectName("_14");
        add1 = new QPushButton(tab_4);
        add1->setObjectName("add1");

        _14->addWidget(add1);

        remove1 = new QPushButton(tab_4);
        remove1->setObjectName("remove1");

        _14->addWidget(remove1);

        spacerItem1 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _14->addItem(spacerItem1);


        gridLayout_3->addLayout(_14, 1, 1, 2, 1);

        abbreviationList = new TextAutoCorrectionWidgets::AutoCorrectionListWidget(tab_4);
        abbreviationList->setObjectName("abbreviationList");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(abbreviationList->sizePolicy().hasHeightForWidth());
        abbreviationList->setSizePolicy(sizePolicy);
        abbreviationList->setSelectionMode(QAbstractItemView::SelectionMode::ExtendedSelection);
        abbreviationList->setSortingEnabled(true);

        gridLayout_3->addWidget(abbreviationList, 2, 0, 1, 1);


        verticalLayout->addLayout(gridLayout_3);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        label_5 = new QLabel(tab_4);
        label_5->setObjectName("label_5");

        gridLayout_2->addWidget(label_5, 0, 0, 1, 1);

        twoUpperLetter = new QLineEdit(tab_4);
        twoUpperLetter->setObjectName("twoUpperLetter");
        twoUpperLetter->setClearButtonEnabled(true);
        twoUpperLetter->setProperty("showClearButton", QVariant(false));

        gridLayout_2->addWidget(twoUpperLetter, 1, 0, 1, 1);

        _16 = new QVBoxLayout();
        _16->setObjectName("_16");
        add2 = new QPushButton(tab_4);
        add2->setObjectName("add2");

        _16->addWidget(add2);

        remove2 = new QPushButton(tab_4);
        remove2->setObjectName("remove2");

        _16->addWidget(remove2);

        spacerItem2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _16->addItem(spacerItem2);


        gridLayout_2->addLayout(_16, 1, 1, 2, 1);

        twoUpperLetterList = new TextAutoCorrectionWidgets::AutoCorrectionListWidget(tab_4);
        twoUpperLetterList->setObjectName("twoUpperLetterList");
        sizePolicy.setHeightForWidth(twoUpperLetterList->sizePolicy().hasHeightForWidth());
        twoUpperLetterList->setSizePolicy(sizePolicy);
        twoUpperLetterList->setSelectionMode(QAbstractItemView::SelectionMode::ExtendedSelection);
        twoUpperLetterList->setSortingEnabled(true);

        gridLayout_2->addWidget(twoUpperLetterList, 2, 0, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        tabWidget->addTab(tab_4, QString());
        tab_1 = new QWidget();
        tab_1->setObjectName("tab_1");
        verticalLayout_3 = new QVBoxLayout(tab_1);
        verticalLayout_3->setObjectName("verticalLayout_3");
        upperCase = new QCheckBox(tab_1);
        upperCase->setObjectName("upperCase");

        verticalLayout_3->addWidget(upperCase);

        upperUpper = new QCheckBox(tab_1);
        upperUpper->setObjectName("upperUpper");

        verticalLayout_3->addWidget(upperUpper);

        ignoreDoubleSpace = new QCheckBox(tab_1);
        ignoreDoubleSpace->setObjectName("ignoreDoubleSpace");

        verticalLayout_3->addWidget(ignoreDoubleSpace);

        autoReplaceNumber = new QCheckBox(tab_1);
        autoReplaceNumber->setObjectName("autoReplaceNumber");

        verticalLayout_3->addWidget(autoReplaceNumber);

        capitalizeDaysName = new QCheckBox(tab_1);
        capitalizeDaysName->setObjectName("capitalizeDaysName");

        verticalLayout_3->addWidget(capitalizeDaysName);

        autoFormatUrl = new QCheckBox(tab_1);
        autoFormatUrl->setObjectName("autoFormatUrl");

        verticalLayout_3->addWidget(autoFormatUrl);

        autoChangeFormat = new QCheckBox(tab_1);
        autoChangeFormat->setObjectName("autoChangeFormat");

        verticalLayout_3->addWidget(autoChangeFormat);

        autoSuperScript = new QCheckBox(tab_1);
        autoSuperScript->setObjectName("autoSuperScript");

        verticalLayout_3->addWidget(autoSuperScript);

        addNonBreakingSpaceInFrench = new QCheckBox(tab_1);
        addNonBreakingSpaceInFrench->setObjectName("addNonBreakingSpaceInFrench");

        verticalLayout_3->addWidget(addNonBreakingSpaceInFrench);

        replaceDoubleQuotesByFrenchQuotes = new QCheckBox(tab_1);
        replaceDoubleQuotesByFrenchQuotes->setObjectName("replaceDoubleQuotesByFrenchQuotes");

        verticalLayout_3->addWidget(replaceDoubleQuotesByFrenchQuotes);

        spacerItem3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(spacerItem3);

        tabWidget->addTab(tab_1, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_5 = new QVBoxLayout(tab_2);
        verticalLayout_5->setObjectName("verticalLayout_5");
        typographicDoubleQuotes = new QCheckBox(tab_2);
        typographicDoubleQuotes->setObjectName("typographicDoubleQuotes");

        verticalLayout_5->addWidget(typographicDoubleQuotes);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        doubleQuoteBeginReplace = new QLabel(tab_2);
        doubleQuoteBeginReplace->setObjectName("doubleQuoteBeginReplace");

        gridLayout_4->addWidget(doubleQuoteBeginReplace, 0, 2, 1, 1);

        doubleQuote1 = new QPushButton(tab_2);
        doubleQuote1->setObjectName("doubleQuote1");
        doubleQuote1->setMaximumSize(QSize(30, 16777215));

        gridLayout_4->addWidget(doubleQuote1, 0, 1, 1, 1);

        doubleQuote2 = new QPushButton(tab_2);
        doubleQuote2->setObjectName("doubleQuote2");
        doubleQuote2->setMaximumSize(QSize(30, 16777215));

        gridLayout_4->addWidget(doubleQuote2, 1, 1, 1, 1);

        label_3 = new QLabel(tab_2);
        label_3->setObjectName("label_3");

        gridLayout_4->addWidget(label_3, 1, 0, 1, 1);

        doubleQuoteEndReplace = new QLabel(tab_2);
        doubleQuoteEndReplace->setObjectName("doubleQuoteEndReplace");

        gridLayout_4->addWidget(doubleQuoteEndReplace, 1, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_2, 0, 3, 2, 1);

        label_2 = new QLabel(tab_2);
        label_2->setObjectName("label_2");

        gridLayout_4->addWidget(label_2, 0, 0, 1, 1);


        verticalLayout_2->addLayout(gridLayout_4);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(50, -1, -1, -1);
        doubleDefault = new QPushButton(tab_2);
        doubleDefault->setObjectName("doubleDefault");

        horizontalLayout_3->addWidget(doubleDefault);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_2->addLayout(horizontalLayout_3);


        verticalLayout_5->addLayout(verticalLayout_2);

        typographicSingleQuotes = new QCheckBox(tab_2);
        typographicSingleQuotes->setObjectName("typographicSingleQuotes");

        verticalLayout_5->addWidget(typographicSingleQuotes);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        gridLayout_5 = new QGridLayout();
        gridLayout_5->setObjectName("gridLayout_5");
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_5->addItem(horizontalSpacer_5, 0, 3, 2, 1);

        singleQuote1 = new QPushButton(tab_2);
        singleQuote1->setObjectName("singleQuote1");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(singleQuote1->sizePolicy().hasHeightForWidth());
        singleQuote1->setSizePolicy(sizePolicy1);
        singleQuote1->setMaximumSize(QSize(30, 16777215));

        gridLayout_5->addWidget(singleQuote1, 0, 1, 1, 1);

        singleQuote2 = new QPushButton(tab_2);
        singleQuote2->setObjectName("singleQuote2");
        sizePolicy1.setHeightForWidth(singleQuote2->sizePolicy().hasHeightForWidth());
        singleQuote2->setSizePolicy(sizePolicy1);
        singleQuote2->setMaximumSize(QSize(30, 16777215));

        gridLayout_5->addWidget(singleQuote2, 1, 1, 1, 1);

        label_8 = new QLabel(tab_2);
        label_8->setObjectName("label_8");

        gridLayout_5->addWidget(label_8, 0, 0, 1, 1);

        simpleQuoteBeginReplace = new QLabel(tab_2);
        simpleQuoteBeginReplace->setObjectName("simpleQuoteBeginReplace");

        gridLayout_5->addWidget(simpleQuoteBeginReplace, 0, 2, 1, 1);

        label_9 = new QLabel(tab_2);
        label_9->setObjectName("label_9");

        gridLayout_5->addWidget(label_9, 1, 0, 1, 1);

        simpleQuoteEndReplace = new QLabel(tab_2);
        simpleQuoteEndReplace->setObjectName("simpleQuoteEndReplace");

        gridLayout_5->addWidget(simpleQuoteEndReplace, 1, 2, 1, 1);


        verticalLayout_4->addLayout(gridLayout_5);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(50, -1, -1, -1);
        singleDefault = new QPushButton(tab_2);
        singleDefault->setObjectName("singleDefault");

        horizontalLayout_4->addWidget(singleDefault);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);


        verticalLayout_4->addLayout(horizontalLayout_4);


        verticalLayout_5->addLayout(verticalLayout_4);

        spacerItem4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(spacerItem4);

        tabWidget->addTab(tab_2, QString());
        tab = new QWidget();
        tab->setObjectName("tab");
        gridLayout_6 = new QGridLayout(tab);
        gridLayout_6->setObjectName("gridLayout_6");
        label_6 = new QLabel(tab);
        label_6->setObjectName("label_6");

        gridLayout_6->addWidget(label_6, 0, 0, 1, 1);

        systemPath = new QLabel(tab);
        systemPath->setObjectName("systemPath");
        systemPath->setTextInteractionFlags(Qt::TextInteractionFlag::LinksAccessibleByMouse|Qt::TextInteractionFlag::TextSelectableByKeyboard|Qt::TextInteractionFlag::TextSelectableByMouse);

        gridLayout_6->addWidget(systemPath, 0, 2, 1, 1);

        horizontalSpacer_6 = new QSpacerItem(473, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_6->addItem(horizontalSpacer_6, 0, 3, 1, 1);

        label_7 = new QLabel(tab);
        label_7->setObjectName("label_7");

        gridLayout_6->addWidget(label_7, 1, 0, 1, 1);

        writablePath = new QLabel(tab);
        writablePath->setObjectName("writablePath");
        writablePath->setTextInteractionFlags(Qt::TextInteractionFlag::LinksAccessibleByMouse|Qt::TextInteractionFlag::TextSelectableByKeyboard|Qt::TextInteractionFlag::TextSelectableByMouse);

        gridLayout_6->addWidget(writablePath, 1, 2, 1, 1);

        horizontalSpacer_7 = new QSpacerItem(473, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_6->addItem(horizontalSpacer_7, 1, 3, 1, 1);

        label_10 = new QLabel(tab);
        label_10->setObjectName("label_10");

        gridLayout_6->addWidget(label_10, 2, 0, 1, 1);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        customSystemPath = new QLineEdit(tab);
        customSystemPath->setObjectName("customSystemPath");

        horizontalLayout_6->addWidget(customSystemPath);

        customSystemPathToolButton = new QToolButton(tab);
        customSystemPathToolButton->setObjectName("customSystemPathToolButton");

        horizontalLayout_6->addWidget(customSystemPathToolButton);


        gridLayout_6->addLayout(horizontalLayout_6, 2, 1, 1, 3);

        label_11 = new QLabel(tab);
        label_11->setObjectName("label_11");

        gridLayout_6->addWidget(label_11, 3, 0, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        customWritablePath = new QLineEdit(tab);
        customWritablePath->setObjectName("customWritablePath");

        horizontalLayout_5->addWidget(customWritablePath);

        customWritablePathToolButton = new QToolButton(tab);
        customWritablePathToolButton->setObjectName("customWritablePathToolButton");

        horizontalLayout_5->addWidget(customWritablePathToolButton);


        gridLayout_6->addLayout(horizontalLayout_5, 3, 1, 1, 3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_6->addItem(verticalSpacer, 4, 1, 1, 1);

        tabWidget->addTab(tab, QString());

        gridLayout->addWidget(tabWidget, 2, 0, 1, 1);

#if QT_CONFIG(shortcut)
        findLabel->setBuddy(find);
        replaceLabel->setBuddy(replace);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(enabledAutocorrection, autocorrectionLanguage);
        QWidget::setTabOrder(autocorrectionLanguage, tabWidget);
        QWidget::setTabOrder(tabWidget, advancedAutocorrection);
        QWidget::setTabOrder(advancedAutocorrection, find);
        QWidget::setTabOrder(find, replace);
        QWidget::setTabOrder(replace, treeWidget);
        QWidget::setTabOrder(treeWidget, addButton);
        QWidget::setTabOrder(addButton, removeButton);
        QWidget::setTabOrder(removeButton, abbreviation);
        QWidget::setTabOrder(abbreviation, abbreviationList);
        QWidget::setTabOrder(abbreviationList, add1);
        QWidget::setTabOrder(add1, remove1);
        QWidget::setTabOrder(remove1, twoUpperLetter);
        QWidget::setTabOrder(twoUpperLetter, twoUpperLetterList);
        QWidget::setTabOrder(twoUpperLetterList, add2);
        QWidget::setTabOrder(add2, remove2);
        QWidget::setTabOrder(remove2, upperCase);
        QWidget::setTabOrder(upperCase, upperUpper);
        QWidget::setTabOrder(upperUpper, ignoreDoubleSpace);
        QWidget::setTabOrder(ignoreDoubleSpace, autoReplaceNumber);
        QWidget::setTabOrder(autoReplaceNumber, capitalizeDaysName);
        QWidget::setTabOrder(capitalizeDaysName, autoFormatUrl);
        QWidget::setTabOrder(autoFormatUrl, autoChangeFormat);
        QWidget::setTabOrder(autoChangeFormat, autoSuperScript);
        QWidget::setTabOrder(autoSuperScript, addNonBreakingSpaceInFrench);
        QWidget::setTabOrder(addNonBreakingSpaceInFrench, replaceDoubleQuotesByFrenchQuotes);
        QWidget::setTabOrder(replaceDoubleQuotesByFrenchQuotes, typographicDoubleQuotes);
        QWidget::setTabOrder(typographicDoubleQuotes, doubleQuote1);
        QWidget::setTabOrder(doubleQuote1, doubleQuote2);
        QWidget::setTabOrder(doubleQuote2, doubleDefault);
        QWidget::setTabOrder(doubleDefault, typographicSingleQuotes);
        QWidget::setTabOrder(typographicSingleQuotes, singleQuote1);
        QWidget::setTabOrder(singleQuote1, singleQuote2);
        QWidget::setTabOrder(singleQuote2, singleDefault);
        QWidget::setTabOrder(singleDefault, customSystemPath);
        QWidget::setTabOrder(customSystemPath, customSystemPathToolButton);
        QWidget::setTabOrder(customSystemPathToolButton, customWritablePath);
        QWidget::setTabOrder(customWritablePath, customWritablePathToolButton);
        QWidget::setTabOrder(customWritablePathToolButton, exportAutoCorrection);
        QWidget::setTabOrder(exportAutoCorrection, importAutoCorrection);

        retranslateUi(AutoCorrectionWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(AutoCorrectionWidget);
    } // setupUi

    void retranslateUi(QWidget *AutoCorrectionWidget)
    {
        enabledAutocorrection->setText(tr2i18n("Enable autocorrection", nullptr));
        label->setText(tr2i18n("Replacement and exception for language:", nullptr));
        exportAutoCorrection->setText(tr2i18n("Export\342\200\246", nullptr));
        importAutoCorrection->setText(tr2i18n("Import\342\200\246", nullptr));
        addButton->setText(tr2i18n("&Add", nullptr));
        removeButton->setText(tr2i18n("Remove", nullptr));
        advancedAutocorrection->setText(tr2i18n("&Enable word replacement", nullptr));
        findLabel->setText(tr2i18n("&Find", nullptr));
        replaceLabel->setText(tr2i18n("&Replace", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = treeWidget->headerItem();
        ___qtreewidgetitem->setText(1, tr2i18n("Replace", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Find", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), tr2i18n("Ad&vanced Autocorrection", nullptr));
        label_4->setText(tr2i18n("Do not treat as the end of a sentence:", nullptr));
        add1->setText(tr2i18n("&Add", nullptr));
        remove1->setText(tr2i18n("R&emove", nullptr));
        label_5->setText(tr2i18n("Accept two uppercase letters in:", nullptr));
        add2->setText(tr2i18n("Add", nullptr));
        remove2->setText(tr2i18n("Remove", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), tr2i18n("E&xceptions", nullptr));
#if QT_CONFIG(whatsthis)
        upperCase->setWhatsThis(tr2i18n("Detect when a new sentence is started and always ensure that the first character is an uppercase character", nullptr));
#endif // QT_CONFIG(whatsthis)
        upperCase->setText(tr2i18n("Convert &first letter of a sentence automatically to uppercase\n"
"(e.g. \"my house. in this town\" to \"my house. In this town\")", nullptr));
#if QT_CONFIG(tooltip)
        upperUpper->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        upperUpper->setWhatsThis(tr2i18n("All words are checked for the common mistake of holding the shift key down a bit too long. If some words must have two uppercase characters, then those exceptions should be added in the 'Exceptions' tab.", nullptr));
#endif // QT_CONFIG(whatsthis)
        upperUpper->setText(tr2i18n("Convert &two uppercase characters to one uppercase and one lowercase character\n"
" (e.g. PErfect to Perfect)", nullptr));
#if QT_CONFIG(whatsthis)
        ignoreDoubleSpace->setWhatsThis(tr2i18n("Make sure that more than one space cannot be typed, as this is a common mistake which is quite hard to find in formatted text.", nullptr));
#endif // QT_CONFIG(whatsthis)
        ignoreDoubleSpace->setText(tr2i18n("&Suppress double spaces", nullptr));
#if QT_CONFIG(whatsthis)
        autoReplaceNumber->setWhatsThis(tr2i18n("Most standard fraction notations will be converted when available", nullptr));
#endif // QT_CONFIG(whatsthis)
        autoReplaceNumber->setText(tr2i18n("Re&place 1/2\342\200\246 with \302\275...", nullptr));
        capitalizeDaysName->setText(tr2i18n("Capitalize name of days", nullptr));
#if QT_CONFIG(whatsthis)
        autoFormatUrl->setWhatsThis(tr2i18n("Detect when a URL (Uniform Resource Locator) is typed and provide formatting that matches the way an Internet browser would show a URL.", nullptr));
#endif // QT_CONFIG(whatsthis)
        autoFormatUrl->setText(tr2i18n("Autoformat &URLs (HTML mode)", nullptr));
#if QT_CONFIG(whatsthis)
        autoChangeFormat->setWhatsThis(tr2i18n("When you use _underline_ or *bold*, the text between the underscores or asterisks will be converted to underlined or bold text.", nullptr));
#endif // QT_CONFIG(whatsthis)
        autoChangeFormat->setText(tr2i18n("Automatically do &bold, strikeout and underline formatting (HTML mode)", nullptr));
        autoSuperScript->setText(tr2i18n("Replace 1st\342\200\246 with 1^st... (HTML mode)", nullptr));
        addNonBreakingSpaceInFrench->setText(tr2i18n("Add non breaking space before specific punctuation marks in French text", nullptr));
        replaceDoubleQuotesByFrenchQuotes->setText(tr2i18n("Replace double quotes by french quotes", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_1), tr2i18n("Si&mple Autocorrection", nullptr));
        typographicDoubleQuotes->setText(tr2i18n("Replace &double quotes with typographical quotes", nullptr));
        doubleQuoteBeginReplace->setText(tr2i18n("By Default", nullptr));
        doubleQuote1->setText(tr2i18n("\"", nullptr));
        doubleQuote2->setText(tr2i18n("\"", nullptr));
        label_3->setText(tr2i18n("Ending Quote:", nullptr));
        doubleQuoteEndReplace->setText(tr2i18n("By Default", nullptr));
        label_2->setText(tr2i18n("Opening Quote:", nullptr));
        doubleDefault->setText(tr2i18n("Default", nullptr));
        typographicSingleQuotes->setText(tr2i18n("Replace &single quotes with typographical quotes", nullptr));
        singleQuote1->setText(tr2i18n("'", nullptr));
        singleQuote2->setText(tr2i18n("'", nullptr));
        label_8->setText(tr2i18n("Opening Quote:", nullptr));
        simpleQuoteBeginReplace->setText(tr2i18n("By Default", nullptr));
        label_9->setText(tr2i18n("Ending Quotes:", nullptr));
        simpleQuoteEndReplace->setText(tr2i18n("By Default", nullptr));
        singleDefault->setText(tr2i18n("Default", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), tr2i18n("Custom &Quotes", nullptr));
        label_6->setText(tr2i18n("System Path:", nullptr));
        systemPath->setText(tr2i18n("TextLabel", nullptr));
        label_7->setText(tr2i18n("Writable Path:", nullptr));
        writablePath->setText(tr2i18n("TextLabel", nullptr));
        label_10->setText(tr2i18n("Custom System Path:", nullptr));
        customSystemPathToolButton->setText(tr2i18n("\342\200\246", nullptr));
        label_11->setText(tr2i18n("Custom Writable Path:", nullptr));
        customWritablePathToolButton->setText(tr2i18n("\342\200\246", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("Path", nullptr));
        (void)AutoCorrectionWidget;
    } // retranslateUi

};

namespace Ui {
    class AutoCorrectionWidget: public Ui_AutoCorrectionWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // AUTOCORRECTIONWIDGET_H

