// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textutils_syntaxhighlighting_debug.h"


Q_LOGGING_CATEGORY(TEXTUTILS_SYNTAXHIGHLIGHTING_LOG, "org.kde.kf.textutils.syntaxhighlighting", QtInfoMsg)

