/*
  SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/
#include "addmcpserverbasewidget.h"

using namespace TextAutoGenerateTextMcpProtocolWidgets;
using namespace Qt::Literals::StringLiterals;
AddMcpServerBaseWidget::AddMcpServerBaseWidget(QWidget *parent)
    : QWidget{parent}
{
}

AddMcpServerBaseWidget::~AddMcpServerBaseWidget() = default;

#include "moc_addmcpserverbasewidget.cpp"
