// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textautocorrectionautocorrect_debug.h"


Q_LOGGING_CATEGORY(TEXTAUTOCORRECTION_AUTOCORRECT_LOG, "org.kde.kf.textautocorrection.autocorrect", QtInfoMsg)

