// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "speechtotext_whisper_debug.h"


Q_LOGGING_CATEGORY(SPEECHTOTEXT_WHISPER_LOG, "org.kde.kf.speechtotext.whisper", QtInfoMsg)

