// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "autogeneratetext_mcpprotocolserverplugin_debug.h"


Q_LOGGING_CATEGORY(AUTOGENERATETEXT_MCPPROTOCOLSERVER_PLUGIN_LOG, "org.kde.kf.autogeneratetext.mcpprotocolserverplugin", QtInfoMsg)

