/*
  SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/
#include "mcpbase.h"

using namespace TextAutoGenerateTextMcpProtocolCore;
McpBase::McpBase(QObject *parent)
    : QObject{parent}
{
}

McpBase::~McpBase() = default;

#include "moc_mcpbase.cpp"
