/****************************************************************************
** Meta object code from reading C++ file 'emojimodelmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ktextaddons-2.1.2/textemoticons/core/emojimodelmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'emojimodelmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17TextEmoticonsCore17EmojiModelManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto TextEmoticonsCore::EmojiModelManager::qt_create_metaobjectdata<qt_meta_tag_ZN17TextEmoticonsCore17EmojiModelManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextEmoticonsCore::EmojiModelManager",
        "usedIdentifierChanged",
        "",
        "lst",
        "excludeEmoticonsChanged",
        "emojiToneChanged",
        "addIdentifier",
        "identifier",
        "emojiModel",
        "TextEmoticonsCore::EmojiModel*",
        "recentIdentifier",
        "emojiTone",
        "EmojiModelManager::EmojiTone",
        "EmojiTone",
        "All",
        "Original",
        "Light",
        "MediumLight",
        "Medium",
        "MediumDark",
        "Dark"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'usedIdentifierChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 },
        }}),
        // Signal 'excludeEmoticonsChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'emojiToneChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'addIdentifier'
        QtMocHelpers::MethodData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'emojiModel'
        QtMocHelpers::PropertyData<TextEmoticonsCore::EmojiModel*>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'recentIdentifier'
        QtMocHelpers::PropertyData<QStringList>(10, QMetaType::QStringList, QMC::DefaultPropertyFlags, 0),
        // property 'emojiTone'
        QtMocHelpers::PropertyData<EmojiModelManager::EmojiTone>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'EmojiTone'
        QtMocHelpers::EnumData<enum EmojiTone>(13, 13, QMC::EnumIsScoped).add({
            {   14, EmojiTone::All },
            {   15, EmojiTone::Original },
            {   16, EmojiTone::Light },
            {   17, EmojiTone::MediumLight },
            {   18, EmojiTone::Medium },
            {   19, EmojiTone::MediumDark },
            {   20, EmojiTone::Dark },
        }),
    };
    return QtMocHelpers::metaObjectData<EmojiModelManager, qt_meta_tag_ZN17TextEmoticonsCore17EmojiModelManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextEmoticonsCore::EmojiModelManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TextEmoticonsCore17EmojiModelManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TextEmoticonsCore17EmojiModelManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17TextEmoticonsCore17EmojiModelManagerE_t>.metaTypes,
    nullptr
} };

void TextEmoticonsCore::EmojiModelManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EmojiModelManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->usedIdentifierChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 1: _t->excludeEmoticonsChanged(); break;
        case 2: _t->emojiToneChanged(); break;
        case 3: _t->addIdentifier((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EmojiModelManager::*)(const QStringList & )>(_a, &EmojiModelManager::usedIdentifierChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EmojiModelManager::*)()>(_a, &EmojiModelManager::excludeEmoticonsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (EmojiModelManager::*)()>(_a, &EmojiModelManager::emojiToneChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<TextEmoticonsCore::EmojiModel**>(_v) = _t->emojiModel(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->recentIdentifier(); break;
        case 2: *reinterpret_cast<EmojiModelManager::EmojiTone*>(_v) = _t->emojiTone(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setEmojiTone(*reinterpret_cast<EmojiModelManager::EmojiTone*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *TextEmoticonsCore::EmojiModelManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextEmoticonsCore::EmojiModelManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TextEmoticonsCore17EmojiModelManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TextEmoticonsCore::EmojiModelManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void TextEmoticonsCore::EmojiModelManager::usedIdentifierChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TextEmoticonsCore::EmojiModelManager::excludeEmoticonsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextEmoticonsCore::EmojiModelManager::emojiToneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
