/****************************************************************************
** Meta object code from reading C++ file 'texttospeech.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ktextaddons-2.1.2/textedittexttospeech/texttospeech.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'texttospeech.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20TextEditTextToSpeech12TextToSpeechE_t {};
} // unnamed namespace

template <> constexpr inline auto TextEditTextToSpeech::TextToSpeech::qt_create_metaobjectdata<qt_meta_tag_ZN20TextEditTextToSpeech12TextToSpeechE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextEditTextToSpeech::TextToSpeech",
        "stateChanged",
        "",
        "TextEditTextToSpeech::TextToSpeech::State",
        "aboutToSynthesize",
        "qsizetype",
        "previousId",
        "currentId",
        "say",
        "text",
        "stop",
        "pause",
        "resume",
        "setRate",
        "rate",
        "setPitch",
        "pitch",
        "setVolume",
        "volume",
        "setLocale",
        "locale"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(TextEditTextToSpeech::TextToSpeech::State)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'aboutToSynthesize'
        QtMocHelpers::SignalData<void(qsizetype, qsizetype)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 5, 7 },
        }}),
        // Slot 'say'
        QtMocHelpers::SlotData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pause'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resume'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setRate'
        QtMocHelpers::SlotData<void(double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setPitch'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 16 },
        }}),
        // Slot 'setVolume'
        QtMocHelpers::SlotData<void(double)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 18 },
        }}),
        // Slot 'setLocale'
        QtMocHelpers::SlotData<void(const QLocale &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QLocale, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextToSpeech, qt_meta_tag_ZN20TextEditTextToSpeech12TextToSpeechE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextEditTextToSpeech::TextToSpeech::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextEditTextToSpeech12TextToSpeechE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextEditTextToSpeech12TextToSpeechE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20TextEditTextToSpeech12TextToSpeechE_t>.metaTypes,
    nullptr
} };

void TextEditTextToSpeech::TextToSpeech::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextToSpeech *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<TextEditTextToSpeech::TextToSpeech::State>>(_a[1]))); break;
        case 1: _t->aboutToSynthesize((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2]))); break;
        case 2: _t->say((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->stop(); break;
        case 4: _t->pause(); break;
        case 5: _t->resume(); break;
        case 6: _t->setRate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setPitch((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->setVolume((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->setLocale((*reinterpret_cast<std::add_pointer_t<QLocale>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextToSpeech::*)(TextEditTextToSpeech::TextToSpeech::State )>(_a, &TextToSpeech::stateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextToSpeech::*)(qsizetype , qsizetype )>(_a, &TextToSpeech::aboutToSynthesize, 1))
            return;
    }
}

const QMetaObject *TextEditTextToSpeech::TextToSpeech::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextEditTextToSpeech::TextToSpeech::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextEditTextToSpeech12TextToSpeechE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TextEditTextToSpeech::TextToSpeech::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void TextEditTextToSpeech::TextToSpeech::stateChanged(TextEditTextToSpeech::TextToSpeech::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TextEditTextToSpeech::TextToSpeech::aboutToSynthesize(qsizetype _t1, qsizetype _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
