// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "speechtotext_google_debug.h"


Q_LOGGING_CATEGORY(SPEECHTOTEXT_GOOGLE_LOG, "org.kde.kf.speechtotext.google", QtInfoMsg)

