// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textaddonswidgets_debug.h"


Q_LOGGING_CATEGORY(TEXTADDONSWIDGETS_LOG, "org.kde.kf.textaddonswidgets", QtInfoMsg)

