/****************************************************************************
** Meta object code from reading C++ file 'textautogeneratemanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ktextaddons-2.1.2/textautogeneratetext/core/textautogeneratemanager.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textautogeneratemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20TextAutoGenerateText23TextAutoGenerateManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto TextAutoGenerateText::TextAutoGenerateManager::qt_create_metaobjectdata<qt_meta_tag_ZN20TextAutoGenerateText23TextAutoGenerateManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextAutoGenerateText::TextAutoGenerateManager",
        "sendMessageRequested",
        "",
        "str",
        "askMessageRequested",
        "TextAutoGenerateText::TextAutoGenerateManager::AskMessageInfo",
        "info",
        "pluginsInitializedDone",
        "downloadErrorOccured",
        "msg",
        "currentChatIdChanged",
        "showArchiveChanged",
        "showMessageId",
        "msgId",
        "chatInProgressChanged",
        "inProgress",
        "needToAddInstances",
        "configChanged",
        "loadEngineDone",
        "fontSizeChanged",
        "downloadModelFinished",
        "modelName",
        "answerGeneratedDone",
        "question",
        "chatListChanged",
        "id",
        "modelListChanged",
        "toolInProgress",
        "numberOfSearchStringChanged",
        "number",
        "currentModelChanged",
        "startOllamaRequested",
        "instanceProcessOk",
        "instanceFailed",
        "errorOccurred",
        "QNetworkReply::NetworkError",
        "e",
        "errorString",
        "discussionListIsEmpty",
        "isEmpty",
        "insertBlockCode",
        "openSavedFileFolderDone",
        "QList<QUrl>",
        "urls",
        "TextAutoGenerateText::TextAutoGenerateManager::FileType",
        "fileType",
        "slotChatListChanged",
        "Categories",
        "Category",
        "Unknown",
        "Tools",
        "Small",
        "Medium",
        "Big",
        "Huge",
        "Multilingual",
        "Code",
        "Math",
        "Vision",
        "Embedding",
        "Reasoning",
        "Cloud",
        "Thinking",
        "FileType",
        "Attachment",
        "File",
        "Pdf"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sendMessageRequested'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'askMessageRequested'
        QtMocHelpers::SignalData<void(const TextAutoGenerateText::TextAutoGenerateManager::AskMessageInfo &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'pluginsInitializedDone'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'downloadErrorOccured'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'currentChatIdChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showArchiveChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showMessageId'
        QtMocHelpers::SignalData<void(const QByteArray &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 13 },
        }}),
        // Signal 'chatInProgressChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'needToAddInstances'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loadEngineDone'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontSizeChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'downloadModelFinished'
        QtMocHelpers::SignalData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Signal 'answerGeneratedDone'
        QtMocHelpers::SignalData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Signal 'chatListChanged'
        QtMocHelpers::SignalData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Signal 'modelListChanged'
        QtMocHelpers::SignalData<void(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Signal 'toolInProgress'
        QtMocHelpers::SignalData<void(const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'numberOfSearchStringChanged'
        QtMocHelpers::SignalData<void(int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 },
        }}),
        // Signal 'currentModelChanged'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startOllamaRequested'
        QtMocHelpers::SignalData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'instanceProcessOk'
        QtMocHelpers::SignalData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'instanceFailed'
        QtMocHelpers::SignalData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QNetworkReply::NetworkError, const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 35, 36 }, { QMetaType::QString, 37 },
        }}),
        // Signal 'discussionListIsEmpty'
        QtMocHelpers::SignalData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 39 },
        }}),
        // Signal 'insertBlockCode'
        QtMocHelpers::SignalData<void(const QString &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'openSavedFileFolderDone'
        QtMocHelpers::SignalData<void(const QList<QUrl> &, TextAutoGenerateText::TextAutoGenerateManager::FileType)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 43 }, { 0x80000000 | 44, 45 },
        }}),
        // Slot 'slotChatListChanged'
        QtMocHelpers::SlotData<void(const QString &)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'Categories'
        QtMocHelpers::EnumData<Categories>(47, 48, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {   49, Category::Unknown },
            {   50, Category::Tools },
            {   51, Category::Small },
            {   52, Category::Medium },
            {   53, Category::Big },
            {   54, Category::Huge },
            {   55, Category::Multilingual },
            {   56, Category::Code },
            {   57, Category::Math },
            {   58, Category::Vision },
            {   59, Category::Embedding },
            {   60, Category::Reasoning },
            {   61, Category::Cloud },
            {   62, Category::Thinking },
        }),
        // enum 'FileType'
        QtMocHelpers::EnumData<enum FileType>(63, 63, QMC::EnumIsScoped).add({
            {   49, FileType::Unknown },
            {   64, FileType::Attachment },
            {   65, FileType::File },
            {   66, FileType::Pdf },
        }),
    };
    return QtMocHelpers::metaObjectData<TextAutoGenerateManager, qt_meta_tag_ZN20TextAutoGenerateText23TextAutoGenerateManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextAutoGenerateText::TextAutoGenerateManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText23TextAutoGenerateManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText23TextAutoGenerateManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20TextAutoGenerateText23TextAutoGenerateManagerE_t>.metaTypes,
    nullptr
} };

void TextAutoGenerateText::TextAutoGenerateManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextAutoGenerateManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sendMessageRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->askMessageRequested((*reinterpret_cast<std::add_pointer_t<TextAutoGenerateText::TextAutoGenerateManager::AskMessageInfo>>(_a[1]))); break;
        case 2: _t->pluginsInitializedDone(); break;
        case 3: _t->downloadErrorOccured((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->currentChatIdChanged(); break;
        case 5: _t->showArchiveChanged(); break;
        case 6: _t->showMessageId((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 7: _t->chatInProgressChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->needToAddInstances(); break;
        case 9: _t->configChanged(); break;
        case 10: _t->loadEngineDone(); break;
        case 11: _t->fontSizeChanged(); break;
        case 12: _t->downloadModelFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->answerGeneratedDone((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->chatListChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->modelListChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->toolInProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->numberOfSearchStringChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->currentModelChanged(); break;
        case 19: _t->startOllamaRequested(); break;
        case 20: _t->instanceProcessOk(); break;
        case 21: _t->instanceFailed(); break;
        case 22: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 23: _t->discussionListIsEmpty((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->insertBlockCode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->openSavedFileFolderDone((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TextAutoGenerateText::TextAutoGenerateManager::FileType>>(_a[2]))); break;
        case 26: _t->slotChatListChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QString & )>(_a, &TextAutoGenerateManager::sendMessageRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const TextAutoGenerateText::TextAutoGenerateManager::AskMessageInfo & )>(_a, &TextAutoGenerateManager::askMessageRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::pluginsInitializedDone, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QString & )>(_a, &TextAutoGenerateManager::downloadErrorOccured, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::currentChatIdChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::showArchiveChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QByteArray & )>(_a, &TextAutoGenerateManager::showMessageId, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(bool )>(_a, &TextAutoGenerateManager::chatInProgressChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::needToAddInstances, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::configChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::loadEngineDone, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::fontSizeChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QString & )>(_a, &TextAutoGenerateManager::downloadModelFinished, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QString & )>(_a, &TextAutoGenerateManager::answerGeneratedDone, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QString & )>(_a, &TextAutoGenerateManager::chatListChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QString & )>(_a, &TextAutoGenerateManager::modelListChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QString & )>(_a, &TextAutoGenerateManager::toolInProgress, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(int )>(_a, &TextAutoGenerateManager::numberOfSearchStringChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::currentModelChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::startOllamaRequested, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::instanceProcessOk, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)()>(_a, &TextAutoGenerateManager::instanceFailed, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(QNetworkReply::NetworkError , const QString & )>(_a, &TextAutoGenerateManager::errorOccurred, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(bool )>(_a, &TextAutoGenerateManager::discussionListIsEmpty, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QString & )>(_a, &TextAutoGenerateManager::insertBlockCode, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextAutoGenerateManager::*)(const QList<QUrl> & , TextAutoGenerateText::TextAutoGenerateManager::FileType )>(_a, &TextAutoGenerateManager::openSavedFileFolderDone, 25))
            return;
    }
}

const QMetaObject *TextAutoGenerateText::TextAutoGenerateManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextAutoGenerateText::TextAutoGenerateManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TextAutoGenerateText23TextAutoGenerateManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TextAutoGenerateText::TextAutoGenerateManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void TextAutoGenerateText::TextAutoGenerateManager::sendMessageRequested(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TextAutoGenerateText::TextAutoGenerateManager::askMessageRequested(const TextAutoGenerateText::TextAutoGenerateManager::AskMessageInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TextAutoGenerateText::TextAutoGenerateManager::pluginsInitializedDone()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TextAutoGenerateText::TextAutoGenerateManager::downloadErrorOccured(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TextAutoGenerateText::TextAutoGenerateManager::currentChatIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TextAutoGenerateText::TextAutoGenerateManager::showArchiveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void TextAutoGenerateText::TextAutoGenerateManager::showMessageId(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void TextAutoGenerateText::TextAutoGenerateManager::chatInProgressChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void TextAutoGenerateText::TextAutoGenerateManager::needToAddInstances()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void TextAutoGenerateText::TextAutoGenerateManager::configChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void TextAutoGenerateText::TextAutoGenerateManager::loadEngineDone()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void TextAutoGenerateText::TextAutoGenerateManager::fontSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void TextAutoGenerateText::TextAutoGenerateManager::downloadModelFinished(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void TextAutoGenerateText::TextAutoGenerateManager::answerGeneratedDone(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void TextAutoGenerateText::TextAutoGenerateManager::chatListChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void TextAutoGenerateText::TextAutoGenerateManager::modelListChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void TextAutoGenerateText::TextAutoGenerateManager::toolInProgress(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void TextAutoGenerateText::TextAutoGenerateManager::numberOfSearchStringChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void TextAutoGenerateText::TextAutoGenerateManager::currentModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void TextAutoGenerateText::TextAutoGenerateManager::startOllamaRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void TextAutoGenerateText::TextAutoGenerateManager::instanceProcessOk()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void TextAutoGenerateText::TextAutoGenerateManager::instanceFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void TextAutoGenerateText::TextAutoGenerateManager::errorOccurred(QNetworkReply::NetworkError _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1, _t2);
}

// SIGNAL 23
void TextAutoGenerateText::TextAutoGenerateManager::discussionListIsEmpty(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void TextAutoGenerateText::TextAutoGenerateManager::insertBlockCode(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void TextAutoGenerateText::TextAutoGenerateManager::openSavedFileFolderDone(const QList<QUrl> & _t1, TextAutoGenerateText::TextAutoGenerateManager::FileType _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1, _t2);
}
QT_WARNING_POP
