/*
  SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/
#include "addmcpprotocolclientdialog.h"
#include <QDialogButtonBox>
#include <QPushButton>
#include <QVBoxLayout>

using namespace TextAutoGenerateTextMcpProtocolWidgets;
AddMcpProtocolClientDialog::AddMcpProtocolClientDialog(QWidget *parent)
    : QDialog(parent)
{
}

AddMcpProtocolClientDialog::~AddMcpProtocolClientDialog()
{
}

#include "moc_addmcpprotocolclientdialog.cpp"
