// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textautogeneratetextmcpprotocol_core_debug.h"


Q_LOGGING_CATEGORY(TEXTAUTOGENERATEMCPPROTOCOLCORE_LOG, "org.kde.kf.textautogeneratetextmcpprotocolcore", QtInfoMsg)

